# 1.2.1 函数

## 函数是什么

函数一词来源于数学，但编程中的「函数」概念，与数学中的函数是有很大不同的，编程中的函数在英文中也有很多不同的叫法。在`BASIC`中叫做`subroutine`(子过程或子程序)，在`Pascal`中叫做`procedure`(过程)和`function`，在`C`中只有`function`，在`Java`里面叫做`method`。

定义: 函数是指将一组语句的集合通过一个名字(函数名)封装起来，要想执行这个函数，只需调用其函数名即可。

## python中函数的定义

`Python`中函数是通过保留字`def`来进行定义。语法格式如下：

```python
def <函数名>([<形式参数>]):
   <函数体>
   return <返回值>
```

函数名可以是任何有效的`Python`标识符；参数是调用该函数时传递给它的值，可以有零个、一个或多个，当传递多个参数时各参数由逗号分隔，当没有参数时也要保留圆括号。

## 函数的参数

### 形参和实参

函数在调用的时候，可以传入参数，有形参和实参，简单点说，形参就是函数接收的参数，而实参就是你实际传入的参数。

**形参**：形参变量只有在被调用时才分配内存单元，在调用结束时，即刻释放所分配的内存单元。因此，形参只在函数内部有效。

**实参**：实参可以是常量、变量、表达式、函数等，无论实参是何种类型的量，在进行函数调用时，它们都必须有确定的值，以便把这些值传送给形参。函数调用结束返回主调用函数后则不能再使用该形参变量。

```python
def calc(x,y):#定义一个函数，参数有x和y，x和y就是形参
   print(x*y)#输出x乘以y的值

calc(5,2)#调用上面定义的函数，5和2就是实参
```

### 函数的四种形参类型

#### 位置参数

位置参数，字面意思也就是按照参数的位置来进行传参，有几个位置参数在调用的时候就要传几个，否则就会报错了。

```python
# name，sex为位置参数/必填参数
def my(name,sex):
    print(name,sex)
    return name

my('wwww','男')
```

#### 默认参数

默认参数就是在定义形参的时候，给函数默认赋一个值，比如说数据库的端口这样的，默认给它一个值，这样就算你在调用的时候没传入这个参数，它也是有值的。

```python
# port=3306为默认值参数
def connect(ip,port=3306):
    print(ip,port)

#如果给一个port值，则传新给的值
connect('118.1.1.1',3307)
#如果不填，则使用默认参数
connect('118.1.1.1')
```

#### 可变参数

可变参数有以下几种特点：

`1`、可变参数用`*`来接收，不是必传的；
`2`、它把传入的元素全部都放到了一个元祖里；
`3`、不显示参数个数，后面想传多少个参数就传多少个，它用在参数比较多的情况下
`4`、如果位置参数、默认值参数、可变参数一起使用的的话，可变参数必须在位置参数和默认值参数后面。

```python
#实例：发送报警短信 参数前面加*代表参数组
def send_sms(*phone_num):
    
    #方法1，返回的是元祖
    print(phone_num)
    #方法2，用下面循环的方法，不打印整个元祖，而是打印每一个元素
    # for p in phone_num:
    #     print(p)


send_sms()# 不传参数
send_sms(150)# 传1个
send_sms(151,152,153)# 传N个

# 执行后结果是：
# ()
# (150,)
# (151, 152, 153)
```

#### 关键字参数

关键字参数有以下几种特点：

`1`、关键字参数使用`**`来接收;
`2`、返回的是字典;
`3`、不限制参数个数，非必传;
`4`、当然也可以和上面的几种一起来使用，如果要一起使用的话，关键字参数必须在最后面。

```python
def send_sms2(**phone_num):
    print(phone_num)

send_sms2()
send_sms2(name='xiaohei',sex='nan')
send_sms2(addr='北京',country='中国',aa='hahaha')

# 执行后结果是：
# {}
# {'name': 'xiaohei', 'sex': 'nan'}
# {'addr': '北京', 'country': '中国', 'aa': 'hahaha'}
```

### 函数的返回值-return语句

每个函数都有返回值，如果没有在函数里面指定返回值的话，在`Python`里面函数执行完之后，默认会返回一个`None`，函数也可以有多个返回值，如果有多个返回值的话，会把返回值都放到一个元组中，返回的是一个元组。

为什么要有返回值呢，是因为在这个函数操作完之后，它的结果在后面的程序里面需要用到。

函数中的返回值使用`return`，函数在遇到`return`就立即结束。

什么样的函数需要`return`，什么样的不需要？下面这个只是显示当前日期，就不需要`return`。例如：

```python
import datetime

def get_today():
    print(datetime.datetime.today())
```

### 局部变量和全局变量

`1.` 局部变量意思就是在局部生效的，出了这个变量的作用域，这个变量就失效了。
`2.` 全局变量的意思就是在整个程序里面都生效的，在程序最前面定义的都是全局变量。
`3.` 全局变量如果要在函数中修改的话，需要加`global`关键字声明，如果是`list`、字典和集合的话，则不需要加`global`关键字，直接就可以修改。
`4.` 任何函数都可以修改，所以尽量少用全局变量，原因`1`不够安全。原因`2`全局变量一直占用内存。

```python
#如果两种变量都有，会先用局部变量，当没有局部变量则用全局变量

#全局变量，在函数的外面
name='wangcan'

def get_name():

    # 声明修改全局变量
    global name

    # 局部变量，定义在函数内部
    name='hailong'
    print('函数里面的name：'+name)

def get_name2():
    print('get_name2：',name)

get_name2()  #第一调用，打印wangcan

get_name()  #第二调用，打印函数内的name:hailong
print('函数外面的name：'+name)# 最后打印,因为有函数声明了全局变量，所以打印的是：hailong


#最后的结果是
# get_name2： wangcan
# 函数里面的name：hailong
# 函数外面的name：hailong
```

### 递归调用

在函数内部，可以调用其他函数。如果一个函数在内部调用自身本身，这个函数就是递归函数。

递归调用的意思就是，在这个函数内部自己调用自己，就有点循环的意思，写个递归，如下：

```python

#递归应用举例
def test1():
    num = int(input('please enter a number：'))
    if num%2==0:#判断输入的数字是不是偶数
       return True #如果是偶数的话，程序就退出了，返回true
    print('不是偶数请重新输入！')
    return test1()#如果不是偶数的话继续调用自己，输入值

print(test1())#调用test
```

递归调用的特性：

`1.` 必须有一个明确的结束条件。

`2.` 每次进入更深一层递归时，问题规模相比上次递归都应有所减少。

`3.` 递归效率不高，递归层次过多会导致栈溢出（在计算机中，函数调用是通过栈（`stack`）这种数据结构实现的，每当进入一个函数调用，栈就会加一层栈帧，每当函数返回，栈就会减一层栈帧。由于栈的大小不是无限的，所以，递归调用的次数过多，会导致栈溢出）。
