
# 2.1.6：结构化数组

## 结构化数组

结构化数组其实就是`ndarrays`，其数据类型是由组成一系列命名字段的简单数据类型组成的，在定义结构化数据时需要指定数据类型。

构建结构化数组的数据类型有多种制定方式，如字典、元组列表：

```python
a=np.array([(b'Rex', 9, 81.), (b'Fido', 7, 77.)],
      dtype=[('name', '<S10'), ('age', '<i4'), ('score', '<f4')]) #通过字典定义数据类型

a=np.array([(b'Rex', 9, 81.), (b'Fido', 7, 77.)],dtype={'names':("name","age","score"),"formats":("<S10","<i4","<f4")}) #通过元组定义数据类型

print(a)
'''
两种方式结果都是一样的
输出:array([(b'Rex', 9, 81.), (b'Fido', 7, 77.)],
      dtype=[('name', '<S10'), ('age', '<i4'), ('score', '<f4')])
'''
```

上面案例中的`S10`表示“长度不超过`10`的字符串”，`i4`表示“`4`个字节整型”，`f4`表示“`4`字节浮点型”。

`numpy`的数据类型如下：

|  数据类型 |描述   |示例   |
| :------------: | :------------: | :------------: |
|`b`   |字节型   |`np.dtype('b')`   |
|`i`   |有符号整型 |`np.dtype('i4')==np.int32`   |
|`u`   |无符号整型 |`np.dtype('u')==np.uint8`   |
|`f`   |浮点型   |`np.dtype('f8')==np.int64`   |
|`c`   |复数浮点型   |`np.dtype('c16')==np.complex128`  |
|`S`、`a`   |字符串   |`np.dtype('SS')`   |
|`U`   |`Unicode`字符串   |`np.dtype('U')==np.str\_` |
| `V`  |原生数据   |`np.dtype('V')==np.void`   |

## 通过文件构造结构化数组

`numpy`通过`loadtxt()`函数读取文件内容，假设有以下文件内容，需要读取文件构造结构化数组：

|  name |age   |score   |
| :------------: | :------------: | :------------: |
|amy   |11   |70   |
|kasa   |10   |   80|
|  baron | 9  |   66|


```python
a=np.loadtxt("data.txt",skiprows=1,dtype=[("name","S10"),("age","int"),("score","float")]) #读取文件并设置数据类型，其中skiprows为跳过第一行

print(a)
'''
输出：[(b'amy', 11, 70.) (b'kasa',  10, 80.) (b'baron', 9, 66.)]
'''
```

## 结构化数组常规操作

结构化数组的方便之处在于，你可以通过**索引**或**名称**查看相应的值，并且可以进行快速的数据处理。

```python
a=np.array([(b'Rex', 10, 81.), (b'Fido', 7, 77.),(b'kasr', 9, 55.)],
      dtype=[('name', 'S10'), ('age', '<i4'), ('score', '<f4')]) #构造结构化数组

print(a[0]) #查看第一行
'''
输出:(b'Rex', 10, 81.)
'''

print(a["age"]>=10) #查看是否有大于等于10岁的
'''
输出:[True False False]
'''

print(a["score"]<60) #查看是否有小于60的
'''
输出:[False False  True]
'''

print(a[a["age"]>=10]) #查看大于等于10岁的信息
'''
输出:[(b'Rex', 10, 81.)]
'''

print(a[a["score"]<60]["name"]) #查看小于60的人的姓名
'''
输出:[b'kasr']
'''
```

注意：尽管这里列举的模式对于简单的操作非常有用，但是这些操作场景也可以用`pandas`的`DataFrame`来实现，并且`DataFrame`更加强大。

