# 3.1.4：Series数据选择

## Series数据选择方法

与`Python`中的字典一样，`Series`对象提供了键值对的映射。

```python
In: import pandas as pd 
In: data = pd.Series([0.25, 0.5, 0.75, 1.0], index=['a', 'b', 'c', 'd'])
In: data["b"]
Out: 0.5
```

还可以用`Python`字典的表达式和方法来检测键/索引和值，也可以像字典一样来修改`Series`对象的值。

```python
In: "a" in data
Out: True
In: data.keys()
Out: Index(['a', 'b', 'c', 'd'], dtype='object')
In: list(data.items())
Out: [('a', 0.25), ('b', 0.5), ('c', 0.75), ('d', 1.0)]
In: data["b"] = 0.05    # 也可以通过此方法来扩展Series
In: data
Out:  a 0.25 
	  b 0.05 
 	 c 0.75 
	  d 1.00 
 	 dtype: float64
```

`Series`对象的可变性是一个非常方便的特性：`Pandas`在底层已经为可能发生的内存布局和数据复制自动决策，用户不需要担心这些问题。

## 将Series看作一堆数组
`Series`对象还具备和`Numpy`数组一样的数组数据选择功能，包括**索引、掩码、花哨索引**等操作，具体示例如下所示：
- 将显示索引作为切片
 *(注意：显示索引切片结果包含最后一个索引，也就是能取到“c”的值)*
 
```python
In: data['a':'c'] 
Out: a 0.25 
 	  b 0.50 
	   c 0.75 
 	  dtype: float64
```
- 将隐式整数索引作为切片
*(注意：隐式索引切片结果不包含最后一个索引)*

```python
In: data[0:2]
Out: a 0.25 
	   b 0.50 
	   dtype: float64
```

- 掩码

```python
In: data[(data > 0.3) & (data < 0.8)]
Out: b 0.50 
	   c 0.75 
 	  dtype: float64
```

- 花哨的索引

```python
In: data[["a","e"]]
Out: a 0.25 
  	 e 1.25 
  	 dtype: float64
```


## 索引器：loc和iloc

这些切片和取值操作非常混乱，假如`Series`对象索引序列为整数时，`data[2]`不会取第三行，而是取索引序列为`2`的那一行，也就是说会优先使用显示索引，而`data[1:3]`这样的切片操作会优先使用隐式索引。

```python
In: data = pd.Series(['a', 'b', 'c'], index=[1, 3, 5])
In: data[1]
Out: "a"
In: data[0:2]
Out: 1 a 
	 3 b 
 	dtype: object
```

正是应为这些整数索引很容易造成混淆，所以`Pandas`提供了一些**索引器(indexer)**属性来作为取值的方法。它们不是`Series`对象的函数方法，而是暴露切片接口的属性。

 - `loc`属性：表示取值和切片都是显示的
 
```python
In: data.loc[1]
Out: "a"
In: data.loc[1:3]
Out: 1 a 
 	   3 b 
 	   dtype: object
```

- `iloc`属性：表示取值和切片都是`Python`形式的隐式索引
- 
```python
In: data.iloc[1]
Out: "b"
In: data.iloc[1:3]
Out: 3 b
 	  5 c
 	  dtype: object
```

`Python`代码的设计原则之一式“显示优于隐式”。使用`loc`和`iloc` 可以让代码更容易维护，可读性更高。特别是在处理整数索引的对象时，我强烈推荐使用这两种索引器。它们既可以让代码阅读和理解起来更容易，也能避免因误用索引或者切片而产生的小`bug`。




