# 4.2.5：绘制三维图

## matplotlib画三维图

要画三维图需要先导入`from mpl_toolkits import mplot3d`。导入这个子模块后，就可以在创建任意一个普通坐标轴的过程中添加`projection='3d'`参数，从而创建一个三维坐标轴。三维图的优点是在`notebook`中可以交互浏览。

```python
from mpl_toolkits import mplot3d
import matplotlib.pyplot as plt

fig = plt.figure()
ax = plt.axes(projection='3d')
plt.show()
```

![](24.jpg)

最基本的三维图是由`(x,y,z)`三维坐标点构成的线图与散点图。与之前普通二维图类似，可以用`ax.plot3D`与`ax.scatter3D`函数来创建。不仅创建方式类似，三维图函数的参数也和二维图函数的参数基本相同。
下面来画一个三角螺旋线并在线上随机分布一些散点：

```python
from mpl_toolkits import mplot3d
import matplotlib.pyplot as plt
import numpy as np

ax = plt.axes(projection='3d')

# 三维线的数据
zline = np.linspace(0, 15, 1000)
xline = np.sin(zline)
yline = np.cos(zline)
ax.plot3D(xline, yline, zline, 'gray')

# 三维散点的数据
zdata = 15 * np.random.random(100)
xdata = np.sin(zdata) + 0.1 * np.random.randn(100)
ydata = np.cos(zdata) + 0.1 * np.random.randn(100)
ax.scatter3D(xdata, ydata, zdata, c=zdata, cmap='Greens');

plt.show()
```

![](25.jpg)

与二维`ax.contour`图形一样，`ax.contour3D`要求所有数据都是二维网格数据的形式，并且由函数计算`z`轴数值。下面用三维正弦函数画三维等高线图：

```python
def f(x, y):
    return np.sin(np.sqrt(x ** 2 + y ** 2))

x = np.linspace(-6, 6, 30)
y = np.linspace(-6, 6, 30)
X, Y = np.meshgrid(x, y)
Z = f(X, Y)

fig = plt.figure()
ax = plt.axes(projection='3d')
ax.contour3D(X, Y, Z, 50, cmap='binary')
ax.set_xlabel('x')
ax.set_ylabel('y')
ax.set_zlabel('z');

plt.show()
```

![](26.jpg)

默认的初始显示角度有时不是最优的，`matplotlib`提供了`view_init`可以调整观察角度与方位角。下面我们把俯仰角调整为`60`度(`x-y`平面的旋转角度)，方位角调整为`35`度(绕`z`轴顺时针旋转`35`度)。

```python
ax.view_init(60,35)
plt.show()
```

![](27.jpg)

其实，也可以在`matplotlib`的交互式后端界面直接通过点击、拖拽图形，实现同样的交互旋转效果。

## 线框图和曲面图

接下来我们将学习线框图和曲面图。它们都是将网格数据映射成三维曲面，得到的三维形状非常容易可视化：

```python
def f(x, y):
    return np.sin(np.sqrt(x ** 2 + y ** 2))
x = np.linspace(-6, 6, 30)
y = np.linspace(-6, 6, 30)
X, Y = np.meshgrid(x, y)
Z = f(X, Y)
fig = plt.figure()

#绘制线框图
ax = plt.axes(projection='3d')
ax.plot_wireframe(X, Y, Z, color='black')
plt.show()

```

![](28.jpg)

曲面图和线框图类似，只不过线框图的每个面都是由多边形构成的。需要注意的是，画曲面图需要二维数据，但可以不是直角坐标系。
```python
#绘制曲面图
ax = plt.axes(projection='3d')
ax.plot_surface(X, Y, Z, rstride=1, cstride=1,
cmap='viridis', edgecolor='none')
plt.show()
```

![](29.jpg)

下面创建一个局部的极坐标网络，当我们把它画成`surface3D`图形时，可以获得一种使用了切片的可视化效果：
```python
def f(x, y):
    return np.sin(np.sqrt(x ** 2 + y ** 2))

r = np.linspace(0, 6, 20)
theta = np.linspace(-0.9 * np.pi, 0.8 * np.pi, 40)
r, theta = np.meshgrid(r, theta)

X = r * np.sin(theta)
Y = r * np.cos(theta)
Z = f(X, Y)
ax = plt.axes(projection='3d')
ax.plot_surface(X, Y, Z, rstride=1, cstride=1,
cmap='viridis', edgecolor='none')
plt.show()
```

![](30.jpg)



