# 4.2.6：曲面三角剖分

## 曲面三角剖分

在某些应用的场景中，之前那些些要求均匀采样的网格数据显得太过严格且不太容易实现。这时就可以使用三角剖分图形了。

```python
def f(x, y):
    return np.sin(np.sqrt(x ** 2 + y ** 2))
theta = 2 * np.pi * np.random.random(1000)
r = 6 * np.random.random(1000)
x = np.ravel(r * np.sin(theta))
y = np.ravel(r * np.cos(theta))
z = f(x, y)
ax = plt.axes(projection='3d')

ax.scatter(x, y, z, c=z, cmap='viridis', linewidth=0.5)
plt.show()
```

![](31.jpg)

可以看到图形中还有许多地方需要修补，这些工作可以由`ax.plot_trisurf`函数完成。它首先找到一组所有点都连接起来的三角形，然后用这些三角形创建曲面。
```python
ax = plt.axes(projection='3d')
ax.plot_trisurf(x, y, z,cmap='viridis', edgecolor='none');

```

![](32.jpg)

虽然结果没有之前用均匀网格画的图完美，但是这种三角剖分方法很灵活，可以创建各种有趣的三维图。

## 莫比乌斯带
莫比乌斯带是把一根纸条扭转 `180` 度后，再把两头粘起来做成的纸带圈。从拓扑学的角度看，莫比乌斯带非常神奇，因为它总共只有一个面！接下来让我们用`matplotlib`的三维功能来画一条莫比乌斯带。绘制的关键是想出它的绘图参数：由于它是一条二维带，因此需要两个内在维度。让我们把一维度定义为`θ`，取值范围为`0~2 π`；另一个维度是`w`，取值范围是`-1~1`，表示莫比乌斯带的宽度：
```python
theta = np.linspace(0, 2 * np.pi, 30)
w = np.linspace(-0.25, 0.25, 8)
w, theta = np.meshgrid(w, theta)
```
有了参数之后，我们必须确定带上每个点的直角坐标 (` x`， `y`， `z` )。
仔细思考一下，我们可能会找到两种旋转关系：一种是圆圈绕着圆心旋转（角度用 `θ` 定义），另一种是莫比乌斯带在自己的坐标轴上旋转（角度用` Φ` 定义）。因此，对于一条莫比乌斯带，我们必然会有环的一半扭转 `180` 度，即 `Δ Φ = Δ θ / 2`。
```python
phi = 0.5 * theta
```
现在用我们的三角学知识将极坐标转换成三维直角坐标。定义每个点到中心的距离（半径）`r`，那么直角坐标 (`z`，`y`，`z` ) 就是：
```python
r = 1 + w * np.cos(phi)
x = np.ravel(r * np.cos(theta))
y = np.ravel(r * np.sin(theta))
z = np.ravel(w * np.sin(phi))
```
最后，要画出莫比乌斯带，还必须确保三角剖分是正确的。最好的实现方法就是首先用基本参数化方法定义三角剖分，然后用`Matplotlib`将这个三角剖分映射到莫比乌斯带的三维空间里，这样就可以画出图形：
```python
from matplotlib.tri import Triangulation
tri = Triangulation(np.ravel(w), np.ravel(theta))
ax = plt.axes(projection='3d')
ax.plot_trisurf(x, y, z, triangles=tri.triangles,
cmap='viridis', linewidths=0.2);
ax.set_xlim(-1, 1); ax.set_ylim(-1, 1); ax.set_zlim(-1, 1);
```

![](33.jpg)

