# 5.3.5 不忘初心---多维缩放

多维缩放（Multiple Dimensional Scaling,MDS）是一种经典的降维方法。其主要思想是保持样本在原空间和低维空间的距离不变。因为距离是样本之间的一个很好的分离属性，降维后，保持距离不变，那么就相当于保持了样本的相对空间关系不变。

## MDS算法思想

假如，一份数据可视化后结果如下：

![](13.jpg)

则将数据进行旋转，数据的特征值发生改变，但是每个点与数据中其他点的距离并没有发生改变：

![](14.jpg)

所以，`MDS`算法认为，在数据样本中，每个样本的每个特征值并不是数据间关系的必要特征，真正的基础特征是每个点与数据集中其他点的**距离**。

假设存在距离矩阵，它的第`i`行第`j`列为$$dist_{ij}$$表示样本`i`到样本`j`之间的距离，我们要通过样本的坐标计算出距离矩阵非常简单。但是，反过来，我们想通过距离矩阵还原出每个样本的坐标就很困难了，而`MDS`算法就是用来解决这个问题的。它可以将一个数据集的距离矩阵还原成一个`D`维坐标来表示数据集。

## MDS算法推导

假设存在`m`个样本，在原始空间中的距离矩阵$$D\in R^{m\times m}$$,其第`i`行`j`列为样本`i`到样本`j`之间的距离。目标是获得样本在$$d$$'维空间中的欧式距离等于原空间的欧氏距离，即：

$$
||z_i-z_j||=dist_{ij}
$$

令$$B=Z^TZ\in R^{m\times m}$$，其中`B`为降维后的内积矩阵，$$b_{ij}=z_i^Tz_j$$，则：

$$
dist_{ij}^2=||z_i-z_j||^2=||z_i||^2+||z_j||^2-2z_i^Tz_j
$$

令降维后的样本被中心化，则：

$$
\sum\limits_{i=1}^mdist_{ij}^2=tr(B)+mb_{jj}
$$

$$
\sum\limits_{j=1}^mdist_{ij}^2=tr(B)+mb_{ii}
$$

$$
\sum\limits_{i=1}^m\sum\limits_{j=1}^mdist_{ij}^2=2mtr(B)
$$

令
$$
dist_{i.}^2=\frac{1}{m}\sum\limits_{j=1}^mdist_{ij}^2...(1)
$$

$$
dist_{.j}^2=\frac{1}{m}\sum\limits_{i=1}^mdist_{ij}^2...(2)
$$

$$
dist_{..}^2=\frac{1}{m^2}\sum\limits_{i=1}^m\sum\limits_{j=1}^mdist_{ij}^2...(3)
$$

则

$$
b_{ij}=-\frac{1}{2}(dist_{ij}^2-dist_{i.}^2-dist_{.j}^2+dist^2_{..})
$$

由此即可通过降维前后保持不变的**距离矩阵**求取**内积矩阵**。

再将**内积矩阵**做特征分解，取`d`个最大的特征值所构成的对角矩阵$$\Lambda$$，并求相应的特征向量矩阵$$V$$，最后计算出`Z`:

$$
Z = V\Lambda^{\frac{1}{2}}
$$

## MDS算法流程

`1`. 计算式子`1`，`2`，`3`
`2`. 计算矩阵`B`
`3`. 对矩阵`B`进行特征分解
`4`. 取`d`个最大特征值所构成的对角矩阵，并求相应的特征向量矩阵
`5`. 计算矩阵`Z`

## sklearn中的多维缩放

在降维时，`MDS`的构造函数中有一个常用的参数可以设置：

- `n_components`：即我们进行`MDS`降维时降到的维数。在降维时需要输入这个参数。


`MDS`类中的`fit_transform`函数用于训练模型并进行降维，返回降维后的数据，`fit_transform`函数有一个向量输入：

- `X`：大小为**[样本数量,特征数量]**的`ndarray`，存放需降维的样本

`MDS`的使用代码如下：

```python
from sklearn.manifold import MDS
mds = MDS(2)
Z = mds.fit_transform(X)
```

如果想动手实现多维缩放算法，并掌握如何使用`sklearn`来解决实际问题，可以尝试进入链接进行实战：https://www.educoder.net/shixuns/vry7x6op/challenges