# 6.2.3 人脸性别识别

## 截取人脸ROI

在进行人脸性别识别时，不需要知道除了人脸之外的信息，所以需要根据上一节中检测到的人脸位置将人脸的`ROI`(感兴趣区域)截取下来。

假设检测到的人脸框为`face_rect`，那么可以根据人脸框得到人脸图像。

```python
x, y, w, h = face_rect
f_h, f_w = gray.shape[:2]
# 防止溢出
x = max(0, x)
y = max(0, y)
w = min(f_w - x, w)
h = min(f_h - y, h)
# 截取图像
face = frame[y: y + h, x: x + w, :]
# 保存图像
cv2.imwrite('face.jpg', face)
```

原图：

![](15.jpg)

提取到的人脸ROI：

![](16.jpg)

## 提取HoG特征

方向梯度直方图（`Histogram of Oriented Gradient, HOG`）是一种在计算机视觉和图像处理中用来进行物体检测的特征描述方法。所谓特征描述方法，就是通过从图像中提取**有用信息**和丢弃无关信息来简化图像，并将信息运用于算法。

特征描述方法从图像中提取有用信息，那么在`HOG`中提取了图像那些有用信息呢？

其实，在图像中，物体的边缘和角落包含了关于物体形状的更多信息。也就是我们平时能看到的物体轮廓。如果能够提取出图像边缘和角落的信息，将非常有利于物体检测。

而对于图像，边缘和角落周围的**梯度**这个特征，变化幅度很大，所以在`HOG`特征描述方法中，使用梯度方向的分布（直方图）作为它的特征。

`OpenCV`中提供了提取`HOG`特征的接口。首先，使用`cv2.HOGDescriptor()`函数声明HOG特征描述方法，代码如下：

```python
hog = cv2.HOGDescriptor()
```

然后计算加载的图像的`HOG`特征值，代码如下：

```python
h = hog.compute(img)
```

完整的代码示例如下：

```python
import cv2
hog = cv2.HOGDescriptor()
img = cv2.imread(sample)
feature = hog.compute(img)
```

## 训练性别分类器

提取了人脸ROI的`HOG`特征后，就可以使用`sklearn`训练性别分类器了。

```python
from sklearn.svm import SVC

clf = SVC()
# hog_features为训练集中人脸ROI的HOG特征
clf.fit(hog_features, label)
pred = clf.predict(test_features)
```
